#!/usr/bin/perl
#
# OnetKonekt 1.0
# codename: CzikHunter DeLuxe
# 
# "Abusers' Hell"
#
# by <krzynio@apcoh.org>
#
# http://onetkonekt.apcoh.org
#
# This program uses:
#
# - Perl TCP Tunnel example by Jochen Wiedmann
#
# - My own perl translation of Onet Authentication Routine (reverse engineered by 
#   <venglin@freebsd.lublin.pl> - see http://www.frasunek.com/onet.php).
#
# 
# Usage:
#
# ./onetkonekt.pl [port to bind to] [password file] (old style)
# ./onetkonekt.pl -c config_file
#
# You can download example configfile from http://onetkonekt.apcoh.org/onet.conf
#
# History:
#
# 1.0   (2004-01-14)    [FEATURE]       - new authorization added
#
# 0.7	(2003-12-16)	[OTHER]		- changed tunnel port to 5015
#					- couple of new features added (mostly in network code)
#					- please do not abuse it!
#
# 0.5	(2003-10-12)	[OTHER]		- changed tunnel port to 5012
#
# 0.4	(2003-09-25)	[FEATURE]	- couple of backend features added
#
# 0.3	(2003-09-19)	[OTHER]		- PLD edition
#
# 0.2	(2003-09-15)	[FEATURE]	- added support for configuration files and ability to listen on specified
#					  interfaces (look at the config file).
#
# 0.11	(2003-09-11)	[FEATURE]	- new feature: you can turn on/off colors support with /QUOTE COLOR OFF/ON.
#
# 0.1	(2003-09-10)	[FEATURE]	- added colors and picture/emoticons support:
#
#						- You can use only 4 different colors (7 - orange, 12 or 2 - blue, 4 - red, 10 - teal/navy)
#						- You can insert emoticons or pictures with \name (\haha for example)
#
# 0.03	(2003-09-09)	[BUG]		- NICK regex fix to allow fancy nicks (reported by gophi).
#
# 0.02	(2003-09-06)  	[FEATURE]	- client password support
#
# 0.01	(dontremember)	[BUG]		- initial release
#
# --------------------------------------------------[ 4pc0hf0r(3 ]-----


my $VERSION='1.0';

use strict;
use IO::Socket ();
use LWP::UserAgent;
use HTTP::Request;
use URI::Escape;


use vars qw($debug $PORT $TOHOST $TOPORT $SERVIP $CLIENTIP $goodpass $listen);
$PORT = shift;
my $arg2 = shift;
my $configfile;
my %cookie;

if ($PORT eq '-c') {
	open FH, $arg2 or die "Cannot open config file: $!";
	my %p;
	while (<FH>){
		chomp $_;
		$_=~ s/( |\r|\t)//sg;
			if ($_ !~ /^#/ && $_ ne ''){
			my ($key, $value)=split /=/, $_;
			$p{$key}=$value;
		}		
	}	
	close FH;
	$PORT=$p{ONET_PORT};
	$listen=$p{ONET_INTERFACE};
	$goodpass=$p{ONET_PASS};

} else {
	if ($arg2 && -e $arg2){
		open FH, $arg2;
		$goodpass=<FH>;
		chomp $goodpass;
		close FH;
	} 
}

$TOHOST = "czat-s.onet.pl";
$TOPORT = 5015;
$| = 1;


$PORT=5015 if (! $PORT);
$listen='0.0.0.0' if (! $listen);

################################################################################################

{

	my %o = ('port' => $PORT,
           'toport' => $TOPORT,
           'tohost' => $TOHOST);


    my $ah = IO::Socket::INET->new('LocalAddr' => $listen,
                                 'LocalPort' => $PORT,
                                 'Reuse' => 1,
                                 'Listen' => 10)
    || die "Failed to bind to local socket: $!";

  $SIG{'CHLD'} = 'IGNORE';
  my $num = 0;
  while (1) {
    my $ch = $ah->accept();
    if (!$ch) {
      print STDERR "Failed to accept: $!\n";
      next;
    }

    ++$num;

    my $pid = fork();
    if (!defined($pid)) {
      print STDERR "Failed to fork: $!\n";
    } elsif ($pid == 0) {
      # This is the child
      $ah->close();
      Run(\%o, $ch, $num);
    } else {
      $ch->close();
    }
  }
}


sub Run {
  my($o, $ch, $num) = @_;
  my $th = IO::Socket::INET->new('PeerAddr' => $o->{'tohost'},
				 'PeerPort' => $o->{'toport'});
	

  $SERVIP=$th->sockhost;
  $CLIENTIP=$ch->peerhost;


  if (!$th) {
    exit 0;
  }

  my $fh;
  my $nick;
  my $pass; 
  my $color=1;

  $ch->autoflush();
  $th->autoflush();

   while ($ch || $th) {


    my $rin = "";

    vec($rin, fileno($ch), 1) = 1 if $ch;
    vec($rin, fileno($th), 1) = 1 if $th;
    my($rout, $eout);
    select($rout = $rin, undef, $eout = $rin, 120);
    if (!$rout  &&  !$eout) {
      print STDERR "Child: Timeout, terminating.\n";
    }
    my $cbuffer = "";
    my $tbuffer = "";

    if ($ch  &&  (vec($eout, fileno($ch), 1)  ||
		  vec($rout, fileno($ch), 1))) {
      my $result = sysread($ch, $tbuffer, 1024);

      if ($tbuffer=~ /^PASS/s && !$pass){
                $pass=$tbuffer;
                $pass=~ s/^.*PASS (.*?)(\r|\n).*$/$1/s;
        }

	

		
      if ($tbuffer=~ /NICK/ && ! $nick) {
	$nick=$tbuffer;
	$nick=~ s/NICK ~/NICK /s;
	$nick=~ s/^.*NICK (.*?)(\r|\n| ).*$/$1/s;
	$tbuffer ='';
	my $res = syswrite($th, "AUTHKEY\n", 8);
	if ($goodpass && $pass ne $goodpass) {
		my $msg="INVALID PASSWORD. PLEASE TRY AGAIN.\r\n";
                my $res = syswrite($ch, $msg, length($msg));
		exit;
	}
      };	

      ### modify VERSION reply...

      $tbuffer=~ s/(NOTICE.*?VERSION)/$1 onetkonekt $VERSION \(http:\/\/onetkonekt.apcoh.org - do not abuse it) [$SERVIP\/$CLIENTIP]/s;
	
      if ($tbuffer=~ /^COLOR/s || $tbuffer=~ /(\r|\n)COLOR/s){
	if ($tbuffer=~ /COLOR ON/s){
		$color=1;  
		my $msg="COLOR IS NOW ON\r\n";
	        my $res = syswrite($ch, $msg, length($msg));

	}
	if ($tbuffer=~ /COLOR OFF/s){
		$color=0;
		 my $msg="COLOR IS NOW OFF\r\n";
                 my $res = syswrite($ch, $msg, length($msg)); 
	};
	$tbuffer=~ s/(\r|\n|)COLOR.*?(\r|\n)//s;
      }

      if ($tbuffer=~ /^USER/s) {$tbuffer=''}

	### kolorki misiu!

	$tbuffer=~ s/\002(.*?)\002/%Fb%$1%Fn%/sg;
	
	$tbuffer=~ s/(\003\d+),\d+/$1/sg;	# wycinam zmiany tla

	$tbuffer=~ s/\0037([^\d])/%Cc86c00%$1/sg;  # pomaranczowy
        $tbuffer=~ s/\00312([^\d])/%C0f2ab1%$1/sg; # niebieski - lub kolor 2 - w drugo stronem
	$tbuffer=~ s/\0032([^\d])/%C0f2ab1%$1/sg; # niebieski - lub kolor 2 - w drugo stronem
        $tbuffer=~ s/\0034([^\d])/%Ce40f0f%$1/sg;  # czerwony
        $tbuffer=~ s/\00310([^\d])/%C1a866e%$1/sg; # navy
        $tbuffer=~ s/\003([^\d])/%C%$1/sg;         # czyszczenie
	$tbuffer=~ s/\\(\w+)/%I$1%/sg;
	
	# wszystkie inne kolorki out
	$tbuffer=~ s/\003\d+/%C%/sg;
	

      if (!defined($result)) {
	print STDERR "Child: Error while reading from client: $!\n";
	exit 0;
      }
      if ($result == 0) {
	exit 0;
      }
    }
    if ($th  &&  (vec($eout, fileno($th), 1)  ||
		  vec($rout, fileno($th), 1))) {
      my $result = sysread($th, $cbuffer, 1024);
      if (!defined($result)) {
	print STDERR "Child: Error while reading from tunnel: $!\n";
	exit 0;
      }
      if ($result == 0) {
	exit 0;
      }

      # sprawdzanie czy mesydz ma numer 006 - autoryzacja onetu	 
	
     if ( get_code($cbuffer) eq '006'){ # autoryzujemy sie w onecie
        my $kod=$cbuffer; $kod=~ s/^.*://;
	$kod=dekode($kod);
	my $uokey=&getuo($nick);
	my $msg="AUTHKEY $kod\nONETAUTH UO $uokey 0 2.2.7\n";
	my $res = syswrite($th, $msg, length($msg));
	$cbuffer='';
	$nick='';
    };


    ### tu trzeba pofiltrowac troche smieci...

	# kolorki -  	%Ce40f0f%
	
	if ($color){
		$cbuffer=~ s/%Cc86c00%/\0037/sg;  # pomaranczowy
		$cbuffer=~ s/%C0f2ab1%/\00312/sg; # niebieski - lub kolor 2 - w drugo stronem
		$cbuffer=~ s/%Ce40f0f%/\0034/sg;  # czerwony
		$cbuffer=~ s/%C1a866e%/\00310/sg; # navy
		$cbuffer=~ s/%C%/\003/sg; 	  # czyszczenie
	}

	$cbuffer=~ s/%C\w+?%//sg;

	# support dla kursywy i pogrubienia 
	if ($color) {		
		$cbuffer=~ s/%Fi/%F/sg;
		$cbuffer=~ s/%Fb(.*?)%/\002/sg;
		$cbuffer=~ s/(\002.*?)%Fn%/$1\002/;
	}

	$cbuffer=~ s/%F.*?%//sg;

	$cbuffer=~ s/%I(\w+?)%/\ <$1> /sg;
	

    ##########################################################	
	
   }

    if ($fh  &&  $tbuffer) {
      (print $fh $tbuffer);
    }

    while (my $len = length($tbuffer)) {
      my $res = syswrite($th, $tbuffer, $len);
      if ($res > 0) {
	$tbuffer = substr($tbuffer, $res);
      } else {
	print STDERR "Child: Failed to write to tunnel: $!\n";
      }
    }
    while (my $len = length($cbuffer)) {
      my $res = syswrite($ch, $cbuffer, $len);
      if ($res > 0) {
	$cbuffer = substr($cbuffer, $res);
      } else {
	print STDERR "Child: Failed to write to tunnel: $!\n";
      }
    }
  }
}


sub dekode {
    my ($s)=@_;
    my $s1;

    my (@f1)=(   14, 1, 48, 40, 34, 30, 52, 59, 2, 59,
		 6, 60, 22, 13, 9, 33, 19, 14, 58, 27,
		 22, 15, 36, 46, 8, 11, 44, 59, 30, 0,
		 23, 45, 2, 10, 23, 37, 41, 13, 34, 43,
		 11, 40, 42, 34, 53, 52, 6, 11, 4, 2,
		 38, 26, 18, 12, 10, 27, 24, 55, 25, 54,
		 55, 49, 38, 58, 59, 0, 33, 39, 14, 5,
		 21, 25, 45, 1, 60, 37, 53, 4
		 );
    
    my (@f2) = ( 25, 43, 43, 47, 41, 36, 32, 9, 44, 20,
		 49, 7, 55, 22, 10, 20, 15, 53, 5, 59,
		 30, 37, 20, 27, 11, 14, 41, 3, 17, 1,
		 23, 42, 45, 4, 28, 24, 41, 60, 34, 23,
		 19, 21, 31, 12, 43, 41, 32, 59, 32, 37,
		 56, 0, 13, 15, 28, 25, 29, 8, 28, 47,
		 9, 51, 27, 55, 55, 55, 18, 34, 54, 52,
		 58, 11, 12, 28, 24, 56, 7, 57
		 );
    my (@f3) = ( 27, 48, 27, 16, 3, 29, 56, 44, 5, 37,
		 45, 60, 59, 47, 58, 38, 29, 27, 46, 5,
		 51, 54, 7, 6, 49, 11, 54, 45, 13, 38,
		 60, 40, 24, 26, 57, 28, 55, 52, 11, 60,
		 28, 57, 58, 25, 42, 55, 2, 10, 21, 48,
		 16, 10, 40, 23, 16, 28, 35, 9, 11, 48,
		 47, 9, 27, 10, 35, 22, 38, 29, 13, 49,
		 28, 41, 45, 25, 4, 25, 19, 6
		 );
    my (@p1) = ( 1, 9, 12, 7, 2, 6, 0, 5, 3, 11,
		 4, 14, 10, 8, 13, 15
		 );
    my (@p2) = ( 12, 5, 1, 4, 15, 0, 2, 11, 10, 14,
		 7, 6, 8, 3, 9, 13
		 );

    my (@ai); my (@ai1);
    
    if (length($s) < 16){
	return "(key to short)";
    }
    

    for(my $i = 0; $i < 16; $i++)
    {
	my $c = ord(substr($s, $i, 1));
	$ai[$i] = ($c> ord('9') ? $c> ord('Z') ? ($c - 97) + 36 : ($c - 65) + 10 : $c - 48);
    }
    
    for(my $j = 0; $j < 16; $j++){
	$ai[$j] = $f1[ $ai[$j] + $j];
    }

    (@ai1)=(@ai);

    for(my $k = 0; $k < 16; $k++) {
	$ai[$k] = ($ai[$k] + $ai1[$p1[$k]]) % 62;
    }
    
    for(my $l = 0; $l < 16; $l++) {
	$ai[$l] = $f2[$ai[$l] + $l];
    }
 
    (@ai1)=(@ai);

    for(my $i1 = 0; $i1 < 16; $i1++)
    {
	$ai[$i1] = ($ai[$i1] + $ai1[$p2[$i1]]) % 62;
    }

    for(my $j1 = 0; $j1 < 16; $j1++)
    {
	$ai[$j1] = $f3[$ai[$j1] + $j1];
    }

    for(my $k1 = 0; $k1 < 16; $k1++)
    {
	my $l1 = $ai[$k1];
	$ai[$k1] = $l1 >= 10 ? $l1>= 36 ? (97 + $l1) - 36 : (65 + $l1) - 10 : 48 + $l1;
	$s1.=chr($ai[$k1]);
    }
    
    return $s1;
}


sub get_code {
    my ($line)=(@_);
    my ($none, $code, $rest)=split(" ", $line);
        return $code;
}

sub getuo {
    my ($nick)=@_;
    %cookie={};
    &getspoof(url=>'http://czat.onet.pl/logowanie.html?CH=Towarzyski');
    &getspoof(url=>'http://czat-s.onet.pl:5007/kropka.gif?CV=3&ST=login-enter');
    &getspoof(url=>'http://kropka.onet.pl/_s/kropka/1?DV=czat');
    &getspoof(url=>'http://czat.onet.pl/logowanie.html',content=>"nick=". $nick . "&pass=&wracaj=&stary_czat=1&CH=Towarzyski&P=5015&submit=Wejd<");
    my $res=&getspoof(url=>'http://czat.onet.pl/applet.html?N='. $nick . '&CH=Towarzyski&');
    $res->{content}=~ s/^.*?<param name="uokey".*?value="(.*?)">.*$/$1/si;
    $res->{content};
}

sub getspoof {
    my (%prm)=(@_);
    my $ua = new LWP::UserAgent( timeout => 30 );
    $ua->agent("Mozilla/5.0 (X11; Unix)");
    my $req;
    
    if ($prm{content}){
	$req  = new HTTP::Request POST => "$prm{url}";
    } else {
        $req  = new HTTP::Request GET => "$prm{url}";
    }
    
    my $cookie;
    foreach (keys %cookie){
	$cookie.="$_=$cookie{$_}; ";
    }
    
    $cookie=~ s/; $//;
    
    $req->header('Cookie' => $cookie);
    $req->content(uri_escape($prm{content})) if ($prm{content});
    
    my $res = $ua->request($req);
    my $out;
    foreach ($res->header('set-cookie')){
        $_=~ s/;.*$//;
        my ($cook, $val)=split /=/, $_;
        $cookie{$cook}=$val;
        $out->{cookie}.="$cook=$val; ";
    }
    
    $out->{cookie}=~ s/; $//;
    
    $out->{content}=$res->content;
    $out;
}


